use List::Util qw[min max];

my $first = 1;

while (<>)
{
	$_ =~ s/\".*//;

	my ($concern,@metrics) = split(/,/);

	if ($first)
	{
		@metrics_names = @metrics;
		pop @metrics_names; # Remove "Bug List"
		$first = 0;
		next;
	}

	my ($sec) = $concern =~ /^(\d+(\.\d+)*)/;
	next if length $sec == 0;

	(@{$all_sections{$sec}}) = @metrics;

	&update_parent_count_sum_range("ALL", @metrics);
}

# Calculate parent stats for leaves only

foreach $sec (keys %all_sections)
{
	# Leaves don't have children
	next if defined $all_sections{$sec . ".1"};

	# Only count leaves when there are more than one
	my $parent = &get_parent($sec);
	next if !defined $all_sections{$parent . ".2"};

	my @metrics = @{$all_sections{$sec}};

	push(@leaf_sections, $sec);

	&update_parent_count_sum_range($parent, @metrics);
}

# Calculate mean

foreach $parent_metric (keys %parent_sum)
{
	my ($parent, $metric_name) = $parent_metric =~ /(.*)_(.*)/;
	my $count = $child_count{$parent};
	next unless $count > 1;

	$parent_mean{$parent_metric} = $parent_sum{$parent_metric} / $count;

	#print "$parent_metric Sum: $parent_sum{$parent_metric} Count: $count Mean: $parent_mean{$parent_metric}\n";
}

# Calculate variance and average deviation across all sections

foreach $sec (sort { &num_cmp($a, $b) } keys %all_sections)
{
	my @metrics = @{$all_sections{$sec}};
	my $count = $child_count{"ALL"};
	&update_parent_variance("ALL", $count, @metrics);
}

# Calculate variance and average deviation for leaves only

foreach $sec (sort { &num_cmp($a, $b) } @leaf_sections)
{
	my @metrics = @{$all_sections{$sec}};

	my $parent = &get_parent($sec);
	my $count = $child_count{$parent};
	next unless $count > 1;

	&update_parent_variance($parent, $count, @metrics);
}

# Display summary statistics for leaves

foreach $parent (sort { &num_cmp($a, $b) } keys %child_count)
{
	my $count = $child_count{$parent};
	next unless $count > 1;
	next if $parent eq "ALL";

	++$families;

	print "Concern: $parent";
	print " ($count Children)\n";

	&output_parent_stats($parent);
}

# Display summary stastistics for all concerns

print "\n";
print "All Concerns (";
print $child_count{"ALL"} . ")\n";
&output_parent_stats("ALL");

print "\n";
print "All Leaf Concern Families ($families)\n";

# Summary of the summary statistics

for($i = 0; $i <= $#metrics_names; ++$i)
{
	printf("  $metrics_names[$i]\tMeanAvg: %7.2f AvgDevAvg: %7.2f StdDevAvg: %7.2f\n",
		$metric_mean_sum{$metrics_names[$i]} / $metric_count{$metrics_names[$i]},
		$metric_avg_dev_sum{$metrics_names[$i]} / $metric_count{$metrics_names[$i]},
		$metric_std_dev_sum{$metrics_names[$i]} / $metric_count{$metrics_names[$i]});
}

sub update_parent_count_sum_range
{
	my ($parent, @metrics) = @_;

	++$child_count{$parent};

	for($i = 0; $i <= $#metrics_names; ++$i)
	{
		my $parent_metric_name = $parent . "_" . $metrics_names[$i];

		$parent_sum{$parent_metric_name} += $metrics[$i];

		if (!defined $parent_min{$parent_metric_name} ||
			$parent_min{$parent_metric_name} > $metrics[$i])
		{
			$parent_min{$parent_metric_name} = $metrics[$i];
		}

		if (!defined $parent_max{$parent_metric_name} ||
			$parent_max{$parent_metric_name} < $metrics[$i])
		{
			$parent_max{$parent_metric_name} = $metrics[$i];
		}

		#print "$sec $metrics_names[$i]=$metrics[$i]\n";
	}

}

sub update_parent_variance
{
	my ($parent, $count, @metrics) = @_;

	for($i = 0; $i <= $#metrics_names; ++$i)
	{
		my $parent_metric_name = $parent . "_" . $metrics_names[$i];

		my $mean = $parent_mean{$parent_metric_name};

		my $dev = $metrics[$i] - $mean;

		my $avg_dev = abs $dev;
		
		$parent_var{$parent_metric_name} += ($dev * $dev) / $count;
		$parent_avg_dev{$parent_metric_name} += $avg_dev / $count;

		#printf("%8s %5s Mean: %7.2f Metric: %6.2f Dev: %8.2f VarAccum: %8.2f\n",
		#	$sec,
		#	$metrics_names[$i],
		#	$mean,
		#	$metrics[$i],
		#	$dev,
		#	$parent_var{$parent_metric_name});
	}
}

sub output_parent_stats
{
	my ($parent) = @_;

	for($i = 0; $i <= $#metrics_names; ++$i)
	{
		my $parent_metric_name = $parent . "_" . $metrics_names[$i];

		my $mean = $parent_mean{$parent_metric_name};
		my $var = $parent_var{$parent_metric_name};
		my $avg_dev = $parent_avg_dev{$parent_metric_name};
		my $std_dev = sqrt $var;
		my $min = $parent_min{$parent_metric_name};
		my $max = $parent_max{$parent_metric_name};

		printf("  $metrics_names[$i]\tRange: %16s Mean: %7.2f AvgDev: %7.2f StdDev: %7.2f\n", 
			sprintf("%.2f-%.2f", $min, $max),
			$mean, 
			$avg_dev, 
			$std_dev);

		next if $parent eq "ALL";

		++$metric_count{$metrics_names[$i]};
		$metric_mean_sum{$metrics_names[$i]} += $mean;
		$metric_avg_dev_sum{$metrics_names[$i]} += $avg_dev;
		$metric_std_dev_sum{$metrics_names[$i]} += $std_dev;
	}
}

sub get_parent
{
	my($sec) = @_;

	my (@parts) = split(/\./, $sec);
	pop @parts;

	my $parent;

	foreach $part (@parts)
	{
		$parent .= "." if length $parent > 0;
		$parent .= $part;
	}

	return $parent;
}

sub num_cmp
{
	my (@partsA) = split(/\./, @_[0]);
	my (@partsB) = split(/\./, @_[1]);

	my $i;

	for($i = 0; $i <= $#partsA && $i <= $#partsB; ++$i)
	{
		my $numA = $partsA[$i];
		my $numB = $partsB[$i];

		#print "$numA - $numB\n";
		
		return ($numA - $numB) if ($numA != $numB);
	}

	return $#partsA - $#partsB;
}
